package se.gawell.setrace.logging;

/*
 *     This file is part of SeTrace.
 *
 *   SeTrace is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   SeTrace is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with SeTrace.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

import java.lang.reflect.Field;
import java.util.Collection;
import java.util.Date;

public class SimpleToStringGenerator implements ToStringGenerator {

	public void generateToString(StringBuilder stringBuilder, Object object) {
		if (object instanceof String) {
			stringBuilder.append("\"").append(object).append("\"");
			return;
		}

		if (object instanceof Collection) {
			@SuppressWarnings("rawtypes")
			Collection collection = (Collection) object;
			stringBuilder.append("{").append(collection.size())
					.append(" elements}");
			return;
		}

		if (shouldBeAppendedDirrectly(object)) {
			stringBuilder.append(object);
			return;
		}

		try {
			boolean firstFieldWritten = false;
			for (Field field : object.getClass().getFields()) {
				if (firstFieldWritten) {
					stringBuilder.append(" ");
				}
				firstFieldWritten = true;
				stringBuilder.append(field.getName()).append("=");
				stringBuilder.append("[");
				generateToString(stringBuilder, field.get(object));
				stringBuilder.append("]");
			}
		} catch (IllegalArgumentException e) {
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		}
	}

	private boolean shouldBeAppendedDirrectly(Object object) {
		if (object == null)
			return true;
		if (object.getClass().isPrimitive())
			return true;
		if (object.getClass().isAssignableFrom(Integer.class))
			return true;

		if (object.getClass().isAssignableFrom(Long.class))
			return true;

		if (object.getClass().isAssignableFrom(Boolean.class))
			return true;

		if (object.getClass().isAssignableFrom(Character.class))
			return true;

		if (object.getClass().isAssignableFrom(Date.class))
			return true;

		if (object.getClass().isEnum())
			return true;

		return false;
	}

}
